@@ -95,6 +95,10 @@ def _anno_parser(node: ast.AST, imports: List[str],
     elif isinstance(node, ast.Tuple):
         return ", ".join(
             _anno_parser(e, imports, disable_from_import) for e in node.elts)
+    elif isinstance(node, ast.List):
+        return ", ".join(
+            _anno_parser(e, imports, disable_from_import) for e in node.elts)
+
     elif isinstance(node, ast.Name):
         return node.id
     elif isinstance(node, ast.Attribute):
@@ -418,6 +422,12 @@ class PybindMethodDecl(object):
         addr = addr_fmt.format(arg_types, self.addr)
         return addr
 
+    def get_python_name(self):
+        if isinstance(self.decl.meta, ConstructorMeta):
+            return "__init__"
+        else:
+            return self.bind_name
+
     def to_string(self) -> str:
         if isinstance(self.decl.meta, ConstructorMeta):
             if self.args:
@@ -725,7 +735,8 @@ def _generate_python_interface_class(cls_name: str,
                                                        PybindPropDecl]],
                                      enum_classes: List[EnumClass],
                                      exist_annos: Dict[str, str],
-                                     parent_name: str = ""):
+                                     parent_import_ns: str = "",
+                                     parent_class_name: str = ""):
     """
     dep_imports
     class xxx:
@@ -766,9 +777,10 @@ def _generate_python_interface_class(cls_name: str,
         decl_codes.append("{}: {}{}".format(prop_decl.get_prop_name(),
                                             prop_anno, default_str))
     for decl in method_decls:
-        if decl.bind_name not in name_to_overloaded:
-            name_to_overloaded[decl.bind_name] = []
-        name_to_overloaded[decl.bind_name].append(decl)
+        decl_python_name = decl.get_python_name()
+        if decl_python_name not in name_to_overloaded:
+            name_to_overloaded[decl_python_name] = []
+        name_to_overloaded[decl_python_name].append(decl)
     for bind_name, over_decls in name_to_overloaded.items():
         if len(over_decls) == 1:
             fmt = "{}def {}({}){}: {}..."
@@ -793,8 +805,8 @@ def _generate_python_interface_class(cls_name: str,
                 res_anno = " -> {}".format(anno)
                 imports.extend(from_imports)
             decl_bind_name = bind_name
-            if isinstance(pydecl.decl.meta, ConstructorMeta):
-                decl_bind_name = "__init__"
+            # if isinstance(pydecl.decl.meta, ConstructorMeta):
+            #     decl_bind_name = "__init__"
 
             arg_names = []  # type: List[str]
             have_default = False  # type: bool
@@ -856,10 +868,9 @@ def _generate_python_interface_class(cls_name: str,
         if not ec.scoped:
             decl_codes.extend(ec_items)
     class_def_string = "class {}:".format(cls_name)
-    if parent_name:
-        class_def_string = "class {}({}):".format(cls_name, parent_name)
-        parent_name_ns = ".".join(parent_name.split(".")[:-1])
-        imports.append("import {}".format(parent_name_ns))
+    if parent_class_name:
+        class_def_string = "class {}({}):".format(cls_name, parent_class_name)
+        imports.append("from {} import {}".format(parent_import_ns, parent_class_name))
     class_block = Block(class_def_string, decl_codes)
     return class_block, imports
 
@@ -872,6 +883,7 @@ class Pybind11SingleClassHandler(ManualClass):
         self.add_include("pybind11/stl.h")
         self.add_include("pybind11/pybind11.h")
         self.add_include("pybind11/numpy.h")
+        self.add_include("pybind11/functional.h")
         self.file_suffix = file_suffix
         self.built = False
 
@@ -1033,12 +1045,18 @@ class Pybind11SplitMain(ParameterizedClass):
                 ns_to_imports[ns] = []
                 ns_to_raw_bind_annos[ns] = []
             parent_name = bind_cu.cu.get_parent_name()
-            if parent_name not in bind_cu_ids:
-                parent_name = ""
-            parent_name = ".".join(parent_name.split("::"))
+            num_level = len(ns.split("."))
+            parent_import_ns = ""
+            parent_class_name = ""
+            if parent_name in bind_cu_ids:
+                parent_name = ".".join(parent_name.split("::"))
+                parent_name_splits = parent_name.split(".")
+                parent_ns = ".".join(parent_name_splits[:-1])
+                parent_import_ns = "." * num_level + parent_ns
+                parent_class_name = parent_name_splits[-1]
             class_block, cls_imports = _generate_python_interface_class(
                 origin_cu.class_name, bind_cu.get_pybind_decls(),
-                origin_cu._enum_classes, exist_annos, parent_name)
+                origin_cu._enum_classes, exist_annos, parent_import_ns, parent_class_name)
             ns_to_imports[ns].extend(cls_imports)
             ns_to_interfaces[ns].append(class_block)
             ns_to_raw_bind_annos[ns].extend(bind_cu.raw_bind_annos)
